#!/usr/bin/env python

"""Tests for `kyman` package."""

import pytest
import yaml

from kyman import cli, iamauthn, kafkacmp


@pytest.fixture()
def cluster():
    cluster_yaml = """
alias: dlas1-mia
url: https://topic-management-service-mia.mp-prod.dlas1.ucloud.int
authn: dlas1
"""
    return yaml.safe_load(cluster_yaml)


def test_get_identity_v2_token(cluster):
    identity_v2_token = iamauthn.get_identity_v2_token(cluster["authn"])
    assert len(identity_v2_token) > 0


def test_process_kobjs():
    kobjs_yaml = """
clusters:
  - dlas1-mia
topics:
- topicName: development.internal.kyman-test.topic-one
  numPartitions: 10
  replicationFactor: 3
  method: POST
acls:
- resourceType: TOPIC
  permissions:
  - READ
  - WRITE
  - DESCRIBE
  principal: svc_james_test
  resourceName: development.internal.kyman-test.topic-one
  method: POST
- resourceType: CONSUMER_GROUP
  permissions:
    - READ
    - WRITE
  principal: svc_james_test
  resourceName: development.kyman-test.consumer-group-one
  enableWildCard: true
  method: POST
- resourceType: TRANSACTIONAL_ID
  permissions:
    - READ
    - WRITE
    - ALL
    - DESCRIBE
    - CREATE
  principal: svc_james_test
  resourceName: kyman-test.transactional-id-one
  enableWildCard: true
  method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    results = kafkacmp.process_kobjs(topic)
    for cluster in results:
        for key in results[cluster]:
            for kobjs in results[cluster][key]:
                assert "errors" not in kobjs and kobjs["response"]["status_code"] == 201
    assert True


def test_process_kobjs_with_missing_key():
    kobjs_yaml = """
clusters:
  - dlas1-mia
topics:
- topicName: development.internal.kyman-test.topic-two
  numPartitions: 10
  method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    results = kafkacmp.process_kobjs(topic)
    for cluster in results:
        for key in results[cluster]:
            for kobjs in results[cluster][key]:
                assert (
                    "errors" in kobjs
                    and kobjs["errors"]
                    == "missing key 'replicationFactor' in the request"
                )
    assert True


def test_list_topics(cluster):
    kobjs_yaml = """
method: GET
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.get_topic(cluster, topic)
    assert (
        result["response"]["status_code"] == 200
        and result["response"]["reason"] == "OK"
    )


def test_get_topic_not_exists(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-two
method: GET
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.get_topic(cluster, topic)
    message = "This server does not host this topic-partition."
    assert (
        result["response"]["status_code"] == 404
        and result["response"]["reason"] == "Not Found"
        and result["response"]["object"]["message"] == message
    )


def test_create_topic(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-two
numPartitions: 11
replicationFactor: 3
method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.create_topic(cluster, topic)
    assert (
        result["response"]["status_code"] == 201
        and result["response"]["reason"] == "Created"
        and result["response"]["object"]["topicName"] == topic["topicName"]
    )


def test_create_topic_with_acl_and_configs(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-three
numPartitions: 12
replicationFactor: 3
aclRequest:
  resourceType: TOPIC
  permissions:
    - READ
  principal: svc_james_test
  resourceName: development.internal.kyman-test.topic-three
configs:
  retention.ms: 4000000000
method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.create_topic(cluster, topic)
    assert (
        result["response"]["status_code"] == 201
        and result["response"]["reason"] == "Created"
        and result["response"]["object"]["topicName"] == topic["topicName"]
    )


def test_get_topic_exists(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-two
method: GET
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.get_topic(cluster, topic)
    assert (
        result["response"]["status_code"] == 200
        and result["response"]["reason"] == "OK"
        and result["response"]["object"]["topicName"] == topic["topicName"]
    )


def test_create_topic_exists(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-two
numPartitions: 12
replicationFactor: 3
method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.create_topic(cluster, topic)
    message = f"Topic '{topic['topicName']}' already exists."
    assert (
        result["response"]["status_code"] == 400
        and result["response"]["reason"] == "Bad Request"
        and result["response"]["object"]["message"] == message
    )


def test_update_topic(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-two
numPartitions: 12
replicationFactor: 3
configs:
  retention.ms: 8000000000
method: PUT
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.update_topic(cluster, topic)
    assert (
        result["response"]["status_code"] == 200
        and result["response"]["reason"] == "OK"
        and result["response"]["object"]["numPartitions"] == 12
        and not result["response"]["object"]["errors"]
    )
    found = False
    for config in result["response"]["object"]["configs"]:
        if config["name"] == "retention.ms" and config["value"] == "8000000000":
            found = True
            break
    assert found


def test_update_topic_with_acl_and_configs(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-two
numPartitions: 12
replicationFactor: 3
aclRequest:
  resourceType: TOPIC
  permissions:
    - READ
  principal: svc_james_test
  resourceName: development.internal.kyman-test.topic-three
configs:
  retention.ms: 4000000000
method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.update_topic(cluster, topic)
    assert (
        result["response"]["status_code"] == 200
        and result["response"]["reason"] == "OK"
        and result["response"]["object"]["topicName"] == topic["topicName"]
    )


def test_create_topic_with_key_missing(cluster):
    kobjs_yaml = """
topicName: development.internal.kyman-test.topic-four
numPartitions: 10
method: POST
"""
    topic = yaml.safe_load(kobjs_yaml)
    try:
        kafkacmp.create_topic(cluster, topic)
    except KeyError as ke:
        assert str(ke) == "'replicationFactor'"


def test_get_acls(cluster):
    kobjs_yaml = """
resourceType: TOPIC
resourceName: development.internal.kyman-test.topic-three
method: GET
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.get_acls(cluster, topic)
    assert (
        result["response"]["status_code"] == 200
        and result["response"]["reason"] == "OK"
        and topic["resourceName"] in result["response"]["url"]
    )


def test_create_acls_for_topic(cluster):
    kobjs_yaml = """
resourceType: TOPIC
permissions:
  - READ
  - WRITE
principal: svc_james_test
host: "*"
resourceName: development.internal.kyman-test.topic-two
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.create_acls(cluster, topic)
    assert (
        result["response"]["status_code"] == 201
        and result["response"]["reason"] == "Created"
        and result["response"]["object"][0]["resourceName"] == topic["resourceName"]
    )
    found_read = False
    found_write = False
    for config in result["response"]["object"]:
        if config["permission"] == "READ":
            found_read = True
            if found_write:
                break
        if config["permission"] == "WRITE":
            found_write = True
            if found_read:
                break
    assert found_read and found_write


def test_delete_acls_for_topic(cluster):
    kobjs_yaml = """
resourceType: TOPIC
permissions:
  - READ
principal: svc_james_test
host: "*"
resourceName: development.internal.kyman-test.topic-two
"""
    topic = yaml.safe_load(kobjs_yaml)
    result = kafkacmp.delete_acls(cluster, topic)
    assert (
        result["response"]["status_code"] == 202
        and result["response"]["reason"] == "Accepted"
        and result["response"]["object"][0]["resourceName"] == topic["resourceName"]
    )


def test_delete_topic(cluster):
    kobjs_yaml = """
topics:
- topicName: development.internal.kyman-test.topic-one
  method: DELETE
- topicName: development.internal.kyman-test.topic-two
  method: DELETE
- topicName: development.internal.kyman-test.topic-three
  method: DELETE
"""
    topics = yaml.safe_load(kobjs_yaml)
    for topic in topics["topics"]:
        result = kafkacmp.delete_topic(cluster, topic)
        assert (
            result["response"]["status_code"] == 202
            and result["response"]["reason"] == "Accepted"
        )


def test_kyman_cli():
    assert cli.main() == 0
